import { cut } from 'jieba-wasm';

function segmentParagraphs(text) {
  return text.split(/[\f\n\r\t\v]+/gm);
}

function segmentSentences(text, options) {
  const paragraphs = segmentParagraphs(text);
  switch (options.lang) {
    case "zh":
      return paragraphs.flatMap(
        (paragraph) => paragraph.match(/[^。？！；]+[。？！；]*[\p{P}]*/gm)
      ).filter((w) => !!w);
    default:
      return paragraphs.flatMap(
        (paragraph) => paragraph.match(/[^.?!;]+[.?!;]*(\x20*)/gm)
      ).filter((w) => !!w);
  }
}

function segmentPhrases(text, options) {
  const paragraphs = segmentParagraphs(text);
  switch (options.lang) {
    case "zh":
      return paragraphs.flatMap(
        (paragraph) => paragraph.match(/[^，。？！；]+[，。？！；]*[\p{P}]*/gm)
      ).filter((w) => !!w);
    default:
      return paragraphs.flatMap(
        (paragraph) => paragraph.match(/[^,.?!;]+[,.?!;]*([\x20]*)/gm)
      ).filter((w) => !!w);
  }
}

function segmentWords(text, options) {
  switch (options.lang) {
    case "zh":
      return cut(text, true);
    default:
      return text.split(/[\x20.,?!;]+/gm).filter((w) => w.length > 0);
  }
}

function createSegmentation(text, options) {
  const { lang } = options;
  switch (options.segmentation) {
    case "paragraphs":
      return segmentParagraphs(text);
    case "sentences":
      return segmentSentences(text, {
        lang
      });
    case "phrases":
      return segmentPhrases(text, {
        lang
      });
    case "words":
      return segmentWords(text, {
        lang
      });
  }
}

export { createSegmentation, segmentParagraphs, segmentPhrases, segmentSentences, segmentWords };
